﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Text.RegularExpressions;
using Microsoft.Maps.MapControl;

namespace OneClickDirections
{
    public partial class MainPage : UserControl
    {
        private PlatformServices.RouteServiceClient routeClient;
        private Color[] colorList = { Colors.Blue, Colors.Green, Colors.Yellow, Colors.Red };
        private int colorIndex = 0;

        public MainPage()
        {
            InitializeComponent();

			routeClient = new PlatformServices.RouteServiceClient("CustomBinding_IRouteService");
            routeClient.CalculateRoutesFromMajorRoadsCompleted += new EventHandler<OneClickDirections.PlatformServices.CalculateRoutesFromMajorRoadsCompletedEventArgs>(routeClient_CalculateRoutesFromMajorRoadsCompleted);

            MyMap.MouseDoubleClick += new EventHandler<MapMouseEventArgs>(MyMap_MouseDoubleClick);

        }

        private void MyMap_MouseDoubleClick(object sender, MapMouseEventArgs e)
        {
            PlatformServices.MajorRoutesRequest majorRouteRequest = new PlatformServices.MajorRoutesRequest();
			majorRouteRequest.Credentials = new Credentials();
			majorRouteRequest.Credentials.ApplicationId = App.Current.Resources["MyCredentials"] as string;


            PlatformServices.Waypoint endPoint = new PlatformServices.Waypoint();
			endPoint.Location = MyMap.ViewportPointToLocation(e.ViewportPoint);

            majorRouteRequest.Destination = endPoint;

            PlatformServices.MajorRoutesOptions options = new PlatformServices.MajorRoutesOptions();
            options.ReturnRoutes = true;
            options.RoutePathType = PlatformServices.RoutePathType.Points;

            majorRouteRequest.Options = options;

            routeClient.CalculateRoutesFromMajorRoadsAsync(majorRouteRequest);
            e.Handled = true;
        }


        private void routeClient_CalculateRoutesFromMajorRoadsCompleted(object sender, OneClickDirections.PlatformServices.CalculateRoutesFromMajorRoadsCompletedEventArgs e)
        {
            ClearRoute();
            int routeCanvasOffset = 5;

            PlatformServices.MajorRoutesResponse routeResponse = e.Result;
            if (routeResponse != null)
            {
                foreach (PlatformServices.RouteResult route in routeResponse.Routes)
                {
                    //Create a new layer for this route 
                    MapLayer routeLayer = new MapLayer();
                    MyMap.Children.Add(routeLayer);

                    colorIndex = (colorIndex + 1) % colorList.Length;

                    // Add Toggle control to toggle the route
                    StackPanel layerToggle = new StackPanel();
                    layerToggle.Orientation = Orientation.Horizontal;
                    CheckBox cb = new CheckBox();
                    cb.IsChecked = true;
                    cb.VerticalAlignment = VerticalAlignment.Center;
                    cb.Tag = routeLayer;
                    cb.Click += new RoutedEventHandler(cb_Click);
                    layerToggle.Children.Add(cb);

                    Rectangle rect = new Rectangle();
                    rect.Width = 30;
                    rect.Height = 30;
                    rect.Fill = new SolidColorBrush(colorList[colorIndex]);
                    rect.Stroke = new SolidColorBrush(Colors.Black);
                    rect.StrokeThickness = 2;
                    layerToggle.Children.Add(rect);
                    LayerPanel.Children.Add(layerToggle);


                    //Draw the polyline for the route
                    MapPolyline routeLine = new MapPolyline();
                    routeLine.Locations = new LocationCollection();
                    Color routeColor = colorList[colorIndex];
                    SolidColorBrush routeBrush = new SolidColorBrush(routeColor);
                    routeLine.Stroke = routeBrush;
                    routeLine.Opacity = 0.65;
                    routeLine.StrokeThickness = 6.0;


                    //Draw the points for the route
                    foreach (Location location in route.RoutePath.Points)
                    {
                        routeLine.Locations.Add(location);
                    }

                    routeLayer.Children.Add(routeLine);


                    Canvas itineraryCanvas = new Canvas();
                    TextBox tb = new TextBox();
                    tb.Width = 400;
                    tb.Height = 150;
                    tb.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
                    tb.HorizontalScrollBarVisibility = ScrollBarVisibility.Auto;
                    tb.IsReadOnly = true;
                    tb.TextWrapping = TextWrapping.Wrap;
                    tb.Background = new SolidColorBrush(colorList[colorIndex]);
                    tb.Foreground = new SolidColorBrush(Colors.Black);
                    tb.Opacity = 0.80;


                    //Create an itinerary canvas
                    string itineraryText = string.Empty;
                    foreach (PlatformServices.ItineraryItem itineraryItem in route.Legs[0].Itinerary)
                    {
                        Ellipse point = new Ellipse();
                        point.Width = 15;
                        point.Height = 15;
                        point.Fill = routeBrush;
                        point.Opacity = 0.65;
                        MapLayer.SetPosition(point, itineraryItem.Location);
                        MapLayer.SetPositionOrigin(point, PositionOrigin.Center);
                        routeLayer.Children.Add(point);

                        
                        foreach (PlatformServices.ItineraryItem item in route.Legs[0].Itinerary)
                        {
                            itineraryText += item.Text + "\r\n";
                        }
                        Regex r1 = new Regex("<[/]*VirtualEarth:\\w*>");
                        itineraryText = r1.Replace(itineraryText, string.Empty);
                    }
                    tb.Text = itineraryText;
                    itineraryCanvas.Children.Add(tb);
                    Canvas.SetLeft(tb, 55);
                    Canvas.SetTop(tb, routeCanvasOffset);

                    routeCanvasOffset += (int)tb.Height + 5;

                    routeLayer.Children.Add(itineraryCanvas);
                }
            }
        }

        void cb_Click(object sender, RoutedEventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if (cb != null)
            {
                MapLayer layer = cb.Tag as MapLayer;
                if (layer != null)
                {
                    if (cb.IsChecked == true)
                    {
                        layer.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        layer.Visibility = Visibility.Collapsed;
                    }
                }
            }
        }

        private void ClearRoute()
        {            
            foreach(UIElement childElement in MyMap.Children)
            {
                MapLayer childLayer = childElement as MapLayer;
                if (childLayer != null)
                {
                    childLayer.Children.Clear();
                }
            }
            LayerPanel.Children.Clear();
        }

    }
}
