﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Globalization;
using System.Windows.Media.Imaging;
using Microsoft.Maps.MapControl;

namespace SearchLayerSample
{
    public partial class MainPage : UserControl
    {
        private Dictionary<string, PlatformServices.SearchResultBase> searchResults = new Dictionary<string, PlatformServices.SearchResultBase>();
        private string[] pinImages = { "pinred.png", "pinblue.png", "pinyellow.png", "pingreen.png" };
        private int pinImageIndex = -1;

        public MainPage()
        {
            InitializeComponent();
            MyMap.ViewChangeOnFrame += new EventHandler<MapEventArgs>(MyMap_ViewChangeOnFrame);
        }


        void MyMap_ViewChangeOnFrame(object sender, MapEventArgs e)
        {
            //PowerLaw scaling
            const double nativeZoomLevel = 17;
            double zoomLevelDiff = nativeZoomLevel - MyMap.ZoomLevel;
            double scaleRatio = 1 / Math.Max(zoomLevelDiff, 1);
            double pinWidth = scaleRatio * 100.0;
            double pinHeight = scaleRatio * 100.0;

            foreach (UIElement childLayer in MyMap.Children)
            {
                MapLayer layer = childLayer as MapLayer;
                if (layer != null)
                {
                    foreach (UIElement child in layer.Children)
                    {
                        Image pin = child as Image;
                        if (pin != null)
                        {
                            pin.Width = (int)pinWidth;
                            pin.Height = (int)pinHeight;
                        }
                    }
                }

            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            string query = Input.Text;
            if (!string.IsNullOrEmpty(query))
            {
                ExecuteSearch(query);
            }
        }

        private void ExecuteSearch(string query)
        {
            //Set up the search request
            PlatformServices.SearchServiceClient searchClient;

            //Create the Service Client
            searchClient = new PlatformServices.SearchServiceClient("BasicHttpBinding_ISearchService");
            searchClient.SearchCompleted += new EventHandler<PlatformServices.SearchCompletedEventArgs>(searchClient_SearchCompleted);

            PlatformServices.SearchRequest request = new PlatformServices.SearchRequest();
            request.Culture = MyMap.Culture;
            request.Query = query;
            request.SearchOptions = new PlatformServices.SearchOptions();
            request.SearchOptions.Count = 25;

            request.ExecutionOptions = new PlatformServices.ExecutionOptions();
            request.ExecutionOptions.SuppressFaults = true;
            // Pass in credentials for web services call.
            //Replace with your own Credentials.
			request.Credentials = new Credentials();
			request.Credentials.ApplicationId = App.Current.Resources["MyCredentials"] as string;

            request.UserProfile = new PlatformServices.UserProfile();
            request.UserProfile.MapView = MyMap.BoundingRectangle;

            //execute the request
            searchClient.SearchAsync(request, query);
        }

        private void searchClient_SearchCompleted(object sender, PlatformServices.SearchCompletedEventArgs e)
        {
            string query = e.UserState as string;

            string outputString;
            try
            {
                if (e.Result.ResponseSummary.StatusCode == PlatformServices.ResponseStatusCode.Success)
                {
                    outputString = string.Format(CultureInfo.InvariantCulture, "Found {0} results", e.Result.ResultSets[0].Results.Count);
                    IList<Location> locations = new List<Location>();
                    
                    if ((e.Result != null) && (e.Result.ResultSets.Count > 0))
                    {
                        pinImageIndex = (pinImageIndex + 1) % pinImages.Length;

                        MapLayer searchLayer = new MapLayer();
                        MyMap.Children.Add(searchLayer);

                        UpdateLayerControl(query, searchLayer, pinImageIndex);

                        foreach (PlatformServices.SearchResultBase result in e.Result.ResultSets[0].Results)
                        {
                            searchResults.Add(result.Id, result);

                            if (result.LocationData.Locations.Count > 0)
                            {
                                //Create a pushpin
								Location location = result.LocationData.Locations[0];
                                UIElement pushpin = CreateSearchResultPushpin(result, pinImageIndex);
                                searchLayer.Children.Add(pushpin);
                                locations.Add(location);
								MapLayer.SetPosition(pushpin, location);
                                MapLayer.SetPositionOrigin(pushpin, PositionOrigin.BottomLeft);
                            }
                        }

                        if (locations.Count > 0)
                        {
                            //If Search results were found, use the bounding area of the results
                            MyMap.SetView(new LocationRect(locations));
                        }
                        else
                        {
                            //Otherwise, if the SearchRegion is set from the service response, go ahead use that
                            LocationRect boundingArea = e.Result.ResultSets[0].SearchRegion.BoundingArea as LocationRect;
                            if (boundingArea != null)
                            {
								MyMap.SetView(boundingArea);
                            }
                        }
                    }
                }
                else
                {
                    outputString = "error searching... status <" + e.Result.ResponseSummary.StatusCode.ToString() + ">";
                }
            }
            catch (Exception)
            {
                outputString = "Exception Raised";
            }

            Output.Text = outputString;
        }

        private void UpdateLayerControl(string searchQuery, MapLayer layer, int index)
        {
            StackPanel sp = new StackPanel();
            sp.Orientation = Orientation.Horizontal;

            CheckBox checkbox = new CheckBox();
            checkbox.IsChecked = true;
            checkbox.VerticalAlignment = VerticalAlignment.Center;
            checkbox.Tag = layer;
            checkbox.Click += new RoutedEventHandler(checkbox_Click);
            sp.Children.Add(checkbox);

            Image image = new Image();
            image.Source = new BitmapImage(new Uri(pinImages[index], UriKind.Relative));
            image.Width=35;
            image.Height=35;
            sp.Children.Add(image);

            TextBlock textBox = new TextBlock();
            textBox.FontSize = 20;
            textBox.Foreground = new SolidColorBrush(Colors.White);
            textBox.Text = searchQuery;
            sp.Children.Add(textBox);

            LayerControl.Children.Add(sp);
        }

        void checkbox_Click(object sender, RoutedEventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if (cb != null)
            {
                MapLayer layer = cb.Tag as MapLayer;
                if (layer != null)
                {
                    if (cb.IsChecked == true)
                    {
                        layer.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        layer.Visibility = Visibility.Collapsed;
                    }
                }
            }
        }

        private UIElement CreateSearchResultPushpin(PlatformServices.SearchResultBase result, int index)
        {
            Image image = new Image();
            image.Source = new BitmapImage(new Uri(pinImages[index], UriKind.Relative));
            image.UseLayoutRounding = false;
            image.Width = 35;
            image.Height = 35;
            return image;
        }

    }
}
